#include <signal.h>
#include <cdef21161.h>
#include <def21161.h>
#include <stdio.h>
asm("#include <def21161.h>");

/*********************************************************************************

	RS232 Test Kernel
	
	This test kernel was authored to run on the ADSP-21161N.  It can, however, be 
	easily changed to run on any SHARC device.
	
	IRQ0 will output the received character from the RS-232 device to the STDIO
	port within VisualDSP++
	
	IRQ1 will send a text message to the RS-232 device
	
	All characters sent from the RS-232 device are returned to the RS-232
	
*********************************************************************************/
	
	

#include "UART_Driver.h"


char	printf_buffer[128];
int		printf_ptr = 0;

void	IRQ0_Handler( int sig_int )
{
	// clear terminal screen
	UartTx_Word(12);

	// output incoming text to STDIO and to RS232	
	printf_buffer[printf_ptr] = 0;
	UartTx_Buffer( printf_buffer, strlen(printf_buffer) );	
	printf("%s",printf_buffer);
	printf_ptr = 0;
}


void	IRQ1_Handler( int sig_int )
{
	char temp_char[128];
	// clear terminal screen
	UartTx_Word(12);

	sprintf(temp_char,"IRQ1!\r\n\r\n");
	UartTx_Buffer( temp_char, strlen(temp_char) );
}


void	UART_RX_Interrupt( int sig_int )
{
	char a;

	asm("TPERIOD = 1000000;TCOUNT = TPERIOD;bit set mode2 TIMEN;");	*(int*) IOFLAG |= 0x2;

	a = UartRx();
	UartTx_Word(a);
	
	printf_buffer[printf_ptr++] = a;
	printf_ptr &=0x7f;
	
}

void	UART_TX_Interrupt( int	sig_int )
{
	asm("TPERIOD = 1000000;TCOUNT = TPERIOD;bit set mode2 TIMEN;");	*(int*) IOFLAG |= 0x1;
}

void	Timer_Handler( int sig_int )
{
	*(int*) IOFLAG &= 0xffffff00;
	asm("bit clr mode2 TIMEN;");

}
main()
{
		
	
	char temp_char[128];

	asm("bit set mode2 FLG0O;");
	asm("bit set flags 1;");
	
	/* Setup hardware interrupts, FLAG LEDs and pushbutton */
	* pIOFLAG = FLG9|FLG8|FLG7|FLG6|FLG5|FLG4|FLG9O|FLG8O|FLG7O|FLG6O|FLG5O|FLG4O;
	
	Setup_UART( 100000000.0, 115200.0 );
	interruptf(UART_RX_INTERRUPT, UART_RX_Interrupt);
	
	// clear terminal screen and send welcome message
	UartTx_Word(12);

	sprintf(temp_char,"Ez-UART-MIDI Test Program!\r\n\r\n");
	UartTx_Buffer( temp_char, strlen(temp_char) );
	
	interruptf(	SIG_IRQ0,	IRQ0_Handler);
	interruptf(	SIG_IRQ1,	IRQ1_Handler);

	asm("TPERIOD = 1000000;TCOUNT = TPERIOD;bit set mode2 TIMEN;");
	interruptf( SIG_TMZ,	Timer_Handler);
	
	sprintf(temp_char,"\r\nHit IRQ0 to dump text that you have typed to both RS-232 terminal \r\nand STDIO output...\r\n ");
	UartTx_Buffer( temp_char, strlen(temp_char) );
		
	while(1)
	{
		asm("idle;");
	}
	
}
